# -*- coding: utf-8 -*-

from django.conf import settings
from organizations.utils import model_field_attr
from organizations.models import get_user_model

User = get_user_model()


ORGS_INVITATION_BACKEND = getattr(settings, 'INVITATION_BACKEND',
        'organizations.backends.defaults.InvitationBackend')

ORGS_REGISTRATION_BACKEND = getattr(settings, 'REGISTRATION_BACKEND',
        'organizations.backends.defaults.RegistrationBackend')

ORGS_EMAIL_LENGTH = model_field_attr(User, 'email', 'max_length')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('organizations', '0002_auto_20150420_1420'),
    ]

    operations = [
        migrations.AddField(
            model_name='organization',
            name='custom_data',
            field=jsonfield.fields.JSONField(default={}, blank=True),
        ),
        migrations.AddField(
            model_name='organization',
            name='custom_settings',
            field=jsonfield.fields.JSONField(default={}, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-

from django import template

from organizations.models import Organization
from organizations.utils import get_users_organizations
from organizations.utils import get_custom_settings_for_current_organization

register = template.Library()


@register.inclusion_tag('organizations/organization_users.html', takes_context=True)
def organization_users(context, org):
    context.update({'organization_users': org.organization_users.all()})
    return context


@register.assignment_tag
def users_organizations(user):
    """
    Returns all organizations, in wich the user is member.
    Use in Template:
    {% load org_tags %}
    {% users_organizations request.user as my_orgs %}
    """
    if not user or not user.is_authenticated():
        return None
    return get_users_organizations(user)

@register.assignment_tag
def custom_settings_for_current_org(request, kw, fallback):
    """
    Access custom settings in templates.

    {% load org_tags %}
    {% custom_settings_for_current_org request 'foo' 'bar' as custom_setting_kw %}
    {{ custom_setting_kw }}
    """
    if not request.user or not request.user.is_authenticated():
        return None
    return get_custom_settings_for_current_organization(request, kw, fallback)


@register.assignment_tag
def orgname_for_slug(slug):
    """
    Returns organizations name for a given slug.
    Use in Template:
    {% load org_tags %}
    {% orgname_for_slug request.current_organization %}
    """
    try:
        return Organization.objects.get(slug=slug).name
    except Organization.DoesNotExist:
        return None



# -*- coding: utf-8 -*-

from django.shortcuts import redirect
from django.core.urlresolvers import reverse 




from organizations.models import Organization
from organizations.utils import set_current_organization_to_session, get_current_organization, skip_request


class OrganizationsMiddleware:
    """
    Simple Middleware that redirects all request to the switch_org view
    if no current_organization is stored in request.session

    Skip the redirect by providing '?org=<slug>' as get parameter
    """

    def process_request(self, request):
        
        if skip_request(request):
            return None
        
        org_slug = request.GET.get('org')
        if org_slug:
            try:
                orgs = Organization.objects.get_for_user(request.user)
                org = orgs.get(slug=org_slug)
            except Organization.DoesNotExist:
                org = None
            except Organization.MultipleObjectsReturned:
                org = None
            if org:
                set_current_organization_to_session(request, org)

        current_organization = get_current_organization(request)
        if not request.path == reverse('organization_switch') and not current_organization and request.user.is_authenticated():
            # skip the redirect and set current_organization if user is member of only one Organization
            url = reverse('organization_switch')
            url += "?next=%s" % request.path
            return redirect(url)

        return None



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-05-31 13:21
from __future__ import unicode_literals

from django.db import migrations
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('organizations', '0003_auto_20150420_1425'),
    ]

    operations = [
        migrations.AlterField(
            model_name='organization',
            name='created',
            field=django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='modified',
            field=django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='slug',
            field=django_extensions.db.fields.AutoSlugField(blank=True, editable=False, help_text='The name in all lowercase, suitable for URL identification', max_length=200, populate_from='name', unique=True),
        ),
        migrations.AlterField(
            model_name='organizationowner',
            name='created',
            field=django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created'),
        ),
        migrations.AlterField(
            model_name='organizationowner',
            name='modified',
            field=django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified'),
        ),
        migrations.AlterField(
            model_name='organizationuser',
            name='created',
            field=django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created'),
        ),
        migrations.AlterField(
            model_name='organizationuser',
            name='modified',
            field=django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
from django.conf import settings
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(default=django.utils.timezone.now, verbose_name='created', editable=False, blank=True)),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(default=django.utils.timezone.now, verbose_name='modified', editable=False, blank=True)),
                ('name', models.CharField(help_text='The name of the organization', max_length=200)),
                ('slug', django_extensions.db.fields.AutoSlugField(populate_from=b'name', editable=False, max_length=200, blank=True, help_text='The name in all lowercase, suitable for URL identification', unique=True)),
                ('is_active', models.BooleanField(default=True)),
            ],
            options={
                'ordering': ['name'],
                'verbose_name': 'organization',
                'verbose_name_plural': 'organizations',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from django.test import TestCase
from django.test.utils import override_settings

from organizations.models import Organization, get_user_model
from organizations.utils import create_organization, model_field_attr

User = get_user_model()


@override_settings(USE_TZ=True)
class OrgManagerTests(TestCase):

    fixtures = ['users.json', 'orgs.json']

    def test_create_organization(self):
        user = User.objects.get(username="dave")
        acme = create_organization(user, "Acme", "acme")
        self.assertTrue(isinstance(acme, Organization))
        self.assertEqual(user, acme.owner.organization_user.user)


class AttributeUtilTests(TestCase):

    def test_present_field(self):
        self.assertTrue(model_field_attr(User, 'username', 'max_length'))

    # def test_absent_field(self):
    #     self.assertRaises(KeyError, model_field_attr, User, 'blahblah',
    #         'max_length')

    # def test_absent_attr(self):
    #     self.assertRaises(AttributeError, model_field_attr, User, 'username',
    #         'mariopoints')



# -*- coding: utf-8 -*-

from django.contrib.auth.decorators import login_required
from compat import url

from organizations import views


urlpatterns = [
    # Organization URLs
    url(r'^$', view=login_required(views.OrganizationList.as_view()),
        name="organization_list"),
    url(r'^switch/$', view=views.switch_org,
        name="organization_switch"),
    url(r'^add/$', view=login_required(views.OrganizationCreate.as_view()),
        name="organization_add"),
    url(r'^(?P<organization_pk>[\d]+)/$',
        view=login_required(views.OrganizationDetail.as_view()),
        name="organization_detail"),
    url(r'^(?P<organization_pk>[\d]+)/edit/$',
        view=login_required(views.OrganizationUpdate.as_view()),
        name="organization_edit"),
    url(r'^(?P<organization_pk>[\d]+)/delete/$',
        view=login_required(views.OrganizationDelete.as_view()),
        name="organization_delete"),

    # Organization user URLs
    url(r'^(?P<organization_pk>[\d]+)/people/$',
        view=login_required(views.OrganizationUserList.as_view()),
        name="organization_user_list"),
    url(r'^(?P<organization_pk>[\d]+)/people/add/$',
        view=login_required(views.OrganizationUserCreate.as_view()),
        name="organization_user_add"),
    url(r'^(?P<organization_pk>[\d]+)/people/(?P<user_pk>[\d]+)/remind/$',
        view=login_required(views.OrganizationUserRemind.as_view()),
        name="organization_user_remind"),
    url(r'^(?P<organization_pk>[\d]+)/people/(?P<user_pk>[\d]+)/$',
        view=login_required(views.OrganizationUserDetail.as_view()),
        name="organization_user_detail"),
    url(r'^(?P<organization_pk>[\d]+)/people/(?P<user_pk>[\d]+)/edit/$',
        view=login_required(views.OrganizationUserUpdate.as_view()),
        name="organization_user_edit"),
    url(r'^(?P<organization_pk>[\d]+)/people/(?P<user_pk>[\d]+)/delete/$',
        view=login_required(views.OrganizationUserDelete.as_view()),
        name="organization_user_delete"),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
from django.conf import settings
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('organizations', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='OrganizationOwner',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(default=django.utils.timezone.now, verbose_name='created', editable=False, blank=True)),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(default=django.utils.timezone.now, verbose_name='modified', editable=False, blank=True)),
                ('organization', models.OneToOneField(related_name='owner', to='organizations.Organization')),
            ],
            options={
                'verbose_name': 'organization owner',
                'verbose_name_plural': 'organization owners',
            },
        ),
        migrations.CreateModel(
            name='OrganizationUser',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(default=django.utils.timezone.now, verbose_name='created', editable=False, blank=True)),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(default=django.utils.timezone.now, verbose_name='modified', editable=False, blank=True)),
                ('is_admin', models.BooleanField(default=False)),
                ('organization', models.ForeignKey(related_name='organization_users', to='organizations.Organization')),
                ('user', models.ForeignKey(related_name='organization_users', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['organization', 'user'],
                'verbose_name': 'organization user',
                'verbose_name_plural': 'organization users',
            },
        ),
        migrations.AddField(
            model_name='organizationowner',
            name='organization_user',
            field=models.OneToOneField(related_name='owned_organization', to='organizations.OrganizationUser'),
        ),
        migrations.AddField(
            model_name='organization',
            name='users',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL, through='organizations.OrganizationUser'),
        ),
        migrations.AlterUniqueTogether(
            name='organizationuser',
            unique_together=set([('user', 'organization')]),
        ),
    ]



# -*- coding: utf-8 -*-

from django.contrib.auth.decorators import user_passes_test, REDIRECT_FIELD_NAME
from organizations.models import OrganizationUser


def organizations_member_required(function=None, redirect_field_name=REDIRECT_FIELD_NAME, login_url=None):
    """
    Decorator for views that checks that the user is logged in if the current user is a member of an organisation,
    redirecting to the log-in page if necessary.
    """

    actual_decorator = user_passes_test(
        lambda u: u.is_authenticated() and bool(OrganizationUser.objects.filter(user=u).count()),
        login_url=login_url,
        redirect_field_name=redirect_field_name
    )
    if function:
        return actual_decorator(function)
    return actual_decorator


def organizations_admin_required(function=None, redirect_field_name=REDIRECT_FIELD_NAME, login_url=None):
    """
    Decorator for views that checks that the user is logged in if the current user is a member of an organisation with admin rights,
    redirecting to the log-in page if necessary.
    """

    actual_decorator = user_passes_test(
        lambda u: u.is_authenticated() and bool(OrganizationUser.objects.filter(user=u, is_admin=True).count()),
        login_url=login_url,
        redirect_field_name=redirect_field_name
    )
    if function:
        return actual_decorator(function)
    return actual_decorator



# -*- coding: utf-8 -*-

import django
from django.db import models


class OrgManager(models.Manager):

    def get_for_user(self, user):
        uid = user.id # bypass SimpleLazyObject error
        if django.VERSION < (1, 7):
            if uid is None:
                return self.get_query_set().none()
            return self.get_query_set().filter(users=user)
        else:
            if uid is None:
                return self.get_queryset().none()
            return self.get_queryset().filter(users=user)


class ActiveOrgManager(OrgManager):
    """
    A more useful extension of the default manager which returns querysets
    including only active organizations
    """

    def get_queryset(self):
        super_ = super(ActiveOrgManager, self)
        if django.VERSION < (1, 7):
            qs = super_.get_query_set()
        else:
            qs = super_.get_queryset()

        return qs.filter(is_active=True)

    if django.VERSION < (1, 7):
        get_query_set = get_queryset
